@@ -3,9 +3,8 @@ import atexit
 from .compile import Logger
 from .terminal.utils import RestartException, handle_exception
 from .terminal import GlobalData, Colors, start
+from .config import VERSION, CONFIG_FILE_NAME
 
-VERSION = 'v1.2.8'
-CONFIG_FILE_NAME = 'jmc_config.json'
 GlobalData().init(VERSION, CONFIG_FILE_NAME)
 
 from . import terminal_commands  # noqa
@@ -15,7 +14,7 @@ logger = Logger(__name__)
 
 def main():
     """Main function"""
-    atexit.register(lambda: print(Colors.EXIT.value))
+    atexit.register(lambda: print(Colors.EXIT.value + "\n"))
     logger.info("Starting session")
     global_data = GlobalData()
     while True:
@@ -29,5 +28,5 @@ def main():
             break
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
