@@ -58,17 +58,19 @@ class RightClickSetup(JMCFunction):
     tag_id_var = "__item_id__"
 
     def call(self) -> str:
-        self.rc_obj = "__rc__" + self.args["idName"][:10]
+        # self.rc_obj = "__rc__" + self.args["idName"][:10]
         func_map = self.datapack.parse_func_map(
             self.raw_args["functionMap"].token, self.tokenizer)
         is_switch = sorted(func_map) == list(range(1, len(func_map) + 1))
 
         id_name = self.args["idName"]
-        self.datapack.add_objective(self.rc_obj, "used:carrot_on_a_stick")
         if self.is_never_used():
-            self.datapack.add_tick_command(
-                f"""execute as @a[scores={{{self.rc_obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name,
-                                                       [f'scoreboard players set @s {self.rc_obj} 0'], 'main')}""")
+            self.add_event(
+                "used:carrot_on_a_stick",
+                self.datapack.call_func(
+                    self.name,
+                    "main"))
+            self.make_empty_private_function("main")
 
         main_func = self.get_private_function("main")
 
@@ -146,18 +148,13 @@ class ItemCreate(JMCFunction):
                 suggestion="Change item_type to carrot_on_a_stick or warped_fungus_on_a_stick")
 
         if on_click:
-            rc_obj = self.rc_obj[self.args["itemType"]]
             item_id = self.datapack.data.get_item_id()
-            self.datapack.add_objective(
-                rc_obj, "used:" + self.args["itemType"])
             if self.is_never_used():
-                self.datapack.add_tick_command(
-                    f"""execute as @a[scores={{{rc_obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name,
-                                                       [
-                                                        f'scoreboard players set @s {rc_obj} 0',
-                                                        f"execute store result score {self.tag_id_var} {DataPack.var_name} run data get entity @s SelectedItem.tag.{self.id_name}",
-                                                        f"execute if score {self.tag_id_var} {DataPack.var_name} matches 1.. run {self.datapack.call_func(self.name, 'found')}"
-                                                        ], "main")}""")
+                self.add_events("used:" + self.args["itemType"], [
+                    f"execute store result score {self.tag_id_var} {DataPack.var_name} run data get entity @s SelectedItem.tag.{self.id_name}",
+                    f"execute if score {self.tag_id_var} {DataPack.var_name} matches 1.. run {self.datapack.call_func(self.name, 'found')}"
+                ]
+                )
                 self.datapack.add_raw_private_function(self.name, [], "found")
 
             found_func = self.get_private_function("found")
@@ -179,6 +176,67 @@ class ItemCreate(JMCFunction):
         return ""
 
 
+@func_property(
+    func_type=FuncType.LOAD_ONLY,
+    call_string="Item.createSpawnEgg",
+    arg_type={
+        "itemId": ArgType.KEYWORD,
+        "mobType": ArgType.KEYWORD,
+        "displayName": ArgType.STRING,
+        "onPlace": ArgType.FUNC,
+        "lore": ArgType.LIST,
+        "nbt": ArgType.JS_OBJECT
+    },
+    name="item_create_spawn_egg",
+    defaults={
+        "lore": "",
+        "nbt": "",
+    }
+)
+class ItemCreateSpawnEgg(JMCFunction):
+    def call(self) -> str:
+        mob_type = self.args["mobType"]
+        spawn_egg = mob_type + "_spawn_egg"
+        item_id = self.args["itemId"]
+        on_place = self.args["onPlace"]
+        name = self.args["displayName"]
+        if self.args["lore"]:
+            lores, lores_tokens = self.datapack.parse_list(
+                self.raw_args["lore"].token, self.tokenizer, TokenType.STRING)
+        else:
+            lores = []
+            lores_tokens = []
+
+        nbt = self.tokenizer.parse_js_obj(
+            self.raw_args["nbt"].token) if self.args["nbt"] else {}
+
+        if "display" in nbt:
+            raise JMCValueError(
+                "display is already inside the nbt",
+                self.token,
+                self.tokenizer)
+
+        lore_ = ",".join(repr(str(FormattedText(lore, lore_token, self.tokenizer, self.datapack, is_default_no_italic=True, is_allow_score_selector=False)))
+                         for lore, lore_token in zip(lores, lores_tokens))
+
+        self.add_event("used:" + spawn_egg, f"""execute as @e[type=marker,tag=__spawn_egg_{item_id}] at @s run {self.datapack.add_raw_private_function(self.name, [
+            "kill @s",
+            on_place
+        ])}""")
+
+        nbt["display"] = Token.empty(f"""{{Name:{repr(
+            str(FormattedText(name, self.raw_args["displayName"].token, self.tokenizer, self.datapack, is_default_no_italic=True, is_allow_score_selector=False))
+            )},Lore:[{lore_}]}},EntityTag:{{id:"minecraft:marker",Tags:["__spawn_egg_{item_id}"]}}""")
+
+        self.datapack.data.item[self.args["itemId"]] = Item(
+            spawn_egg,
+            self.datapack.token_dict_to_raw_js_object(nbt, self.tokenizer),
+            nbt
+        )
+
+        return ""
+
+
 @func_property(
     func_type=FuncType.LOAD_ONLY,
     call_string="Item.createSign",
@@ -200,32 +258,34 @@ class ItemCreate(JMCFunction):
     }
 )
 class ItemCreateSign(JMCFunction):
-    _variants = {"oak", "spruce", "birch", "jungle",
+    _VARIANTS = {"oak", "spruce", "birch", "jungle",
                  "acacia", "dark_oak", "crimson", "warped"}
 
     def call(self) -> str:
         variant = self.args["variant"]
 
-        if variant not in self._variants:
+        if variant not in self._VARIANTS:
             raise JMCValueError(
                 f"Unrecognized wood variant for sign ({variant})",
                 self.raw_args["variant"].token,
                 self.tokenizer,
-                suggestion=f"Available variants are {' '.join(repr(i) for i in self._variants)}")
+                suggestion=f"Available variants are {' '.join(repr(i) for i in self._VARIANTS)}")
 
         on_click = self.args["onClick"]
         name = self.args["displayName"]
         if self.args["lore"]:
-            lores = self.datapack.parse_list(
+            lores, lores_tokens = self.datapack.parse_list(
                 self.raw_args["lore"].token, self.tokenizer, TokenType.STRING)
         else:
             lores = []
+            lores_tokens = []
 
         if self.args["texts"]:
-            texts = self.datapack.parse_list(
+            texts, texts_tokens = self.datapack.parse_list(
                 self.raw_args["texts"].token, self.tokenizer, TokenType.STRING)
         else:
             texts = []
+            texts_tokens = []
 
         if len(texts) > 4:
             raise JMCValueError(
@@ -233,6 +293,7 @@ class ItemCreateSign(JMCFunction):
                 self.raw_args["texts"].token,
                 self.tokenizer)
         texts.extend([''] * (4 - len(texts)))
+        texts_tokens.extend([Token.empty()] * (4 - len(texts_tokens)))
 
         nbt = self.tokenizer.parse_js_obj(
             self.raw_args["nbt"].token) if self.args["nbt"] else {}
@@ -243,10 +304,10 @@ class ItemCreateSign(JMCFunction):
                 self.token,
                 self.tokenizer)
 
-        lore_ = ",".join(repr(str(FormattedText(lore, self.raw_args["lore"].token, self.tokenizer, self.datapack, is_default_no_italic=True, is_allow_score_selector=False)))
-                         for lore in lores)
-        formatted_texts_ = [FormattedText(text, self.raw_args["texts"].token, self.tokenizer, self.datapack) if text else FormattedText.empty(self.tokenizer, self.datapack)
-                            for text in texts]
+        lore_ = ",".join(repr(str(FormattedText(lore, lore_token, self.tokenizer, self.datapack, is_default_no_italic=True, is_allow_score_selector=False)))
+                         for lore, lore_token in zip(lores, lores_tokens))
+        formatted_texts_ = [FormattedText(text, text_token, self.tokenizer, self.datapack) if text else FormattedText.empty(self.tokenizer, self.datapack)
+                            for text, text_token in zip(texts, texts_tokens)]
         if on_click:
             formatted_texts_[0].add_key(
                 "clickEvent", {
@@ -273,27 +334,14 @@ class ItemCreateSign(JMCFunction):
     func_type=FuncType.LOAD_ONLY,
     call_string="Player.onEvent",
     arg_type={
-        "criteria": ArgType.STRING,
+        "criteria": ArgType.KEYWORD,
         "function": ArgType.FUNC,
     },
     name="player_on_event"
 )
 class PlayerOnEvent(JMCFunction):
     def call(self) -> str:
-        objective = f"on_event{self.datapack.get_count('on_event')}"
-        if not self.is_never_used(parameters=[self.args['criteria']]):
-            raise JMCValueError(
-                f"{self.args['criteria']} criteria was already used.",
-                self.raw_args["criteria"].token,
-                self.tokenizer)
-        self.datapack.add_load_command(
-            f"scoreboard objectives add {objective} {self.args['criteria']}"
-        )
-        count = self.datapack.get_count(self.name)
-        func = self.datapack.add_raw_private_function(
-            self.name, [f"scoreboard players set @s {objective} 0", self.args['function']], count=count)
-        self.datapack.add_tick_command(
-            f"execute as @a[scores={{{objective}=1..}}] at @s run {func}")
+        self.add_event(self.args['criteria'], self.args['function'])
         return ""
 
 
@@ -586,7 +634,7 @@ class GUITemplate(JMCFunction):
     }
 
     def call(self) -> str:
-        template = self.datapack.parse_list(
+        template, _ = self.datapack.parse_list(
             self.raw_args["template"].token, self.tokenizer, TokenType.STRING)
         name = convention_jmc_to_mc(
             self.raw_args["name"].token, self.tokenizer)
@@ -660,8 +708,13 @@ class GUIRegisters(JMCFunction):
                 self.raw_args["id"].token,
                 self.tokenizer)
 
-        item_strings = self.datapack.parse_list(self.raw_args["items"].token,
-                                                self.tokenizer, TokenType.KEYWORD)
+        item_strings, item_strings_tokens = self.datapack.parse_list(self.raw_args["items"].token,
+                                                                     self.tokenizer, TokenType.KEYWORD)
+        if not item_strings:
+            raise JMCValueError(
+                f"GUI Template expected non-empty 'items' argument",
+                self.raw_args["items"].token,
+                self.tokenizer)
 
         items: list[Item] = []
 
@@ -670,7 +723,8 @@ class GUIRegisters(JMCFunction):
                 f"gui/{name}/on_click", self.args["onClick"])
             interactive_id = self.datapack.get_count(
                 f"{self.name}/{name}")
-            for item_str in item_strings:
+            for item_str, item_str_token in zip(
+                    item_strings, item_strings_tokens):
                 if item_str not in self.datapack.data.item:
                     raise JMCValueError(
                         f'Item id: \'{item_str}\' is not defined.',
@@ -679,7 +733,7 @@ class GUIRegisters(JMCFunction):
                         suggestion=f"Use Item.create to make this item BEFORE using {self.call_string}"
                     )
                 items.append(self.create_new_item(self.datapack.data.item[item_str], modify_nbt={
-                    "__gui__": Token.empty(f"{{interactive_id:{interactive_id},name:{repr(name)}}}")}))
+                    "__gui__": Token.empty(f"{{interactive_id:{interactive_id},name:{repr(name)}}}")}, error_token=item_str_token))
 
             self.set_items(
                 name,
@@ -689,7 +743,8 @@ class GUIRegisters(JMCFunction):
                 interactive_id,
                 on_click)
         else:
-            for item_str in item_strings:
+            for item_str, item_str_token in zip(
+                    item_strings, item_strings_tokens):
                 if item_str not in self.datapack.data.item:
                     raise JMCValueError(
                         f'Item id: \'{item_str}\' is not defined.',
@@ -698,7 +753,7 @@ class GUIRegisters(JMCFunction):
                         suggestion=f"Use Item.create to make this item BEFORE using {self.call_string}"
                     )
                 items.append(self.create_new_item(self.datapack.data.item[item_str], modify_nbt={
-                    "__gui__": Token.empty(f"{{name:{repr(name)}}}")}))
+                    "__gui__": Token.empty(f"{{name:{repr(name)}}}")}, error_token=item_str_token))
             self.set_items(
                 name,
                 self.args["id"],
@@ -929,6 +984,42 @@ class GUICreate(JMCFunction):
         return ""
 
 
+@func_property(
+    func_type=FuncType.LOAD_ONLY,
+    call_string="Team.add",
+    arg_type={
+        "team": ArgType.KEYWORD,
+        "displayName": ArgType.STRING,
+        "properties": ArgType.JS_OBJECT
+    },
+    name="team_add",
+    defaults={
+        "displayName": "",
+        "properties": "",
+    }
+)
+class TeamAdd(JMCFunction):
+    def call(self) -> str:
+        command = f'team add {self.args["team"]}'
+        if self.args["team"] in self.datapack.data.teams:
+            team = self.datapack.data.teams[self.args["team"]]
+            raise JMCValueError(
+                f"Team {self.args['team']} was already defined",
+                self.raw_args["team"].token,
+                self.tokenizer, suggestion=f"It was defined at line {team[1].line} col {team[1].col}")
+        self.datapack.data.teams[self.args["team"]] = (
+            self.args["displayName"], self.raw_args["team"].token)
+        if self.args["displayName"]:
+            command += ' ' + self.format_text("displayName")
+
+        properties = self.tokenizer.parse_js_obj(
+            self.raw_args["properties"].token) if self.args["properties"] else {}
+        for key, value in properties.items():
+            command += f"\nteam modify {self.args['team']} {key} {value.string}"
+
+        return command
+
+
 # @ func_property(
 #     func_type=FuncType.load_only,
 #     call_string='Debug.track',
