@@ -5,7 +5,7 @@ from typing import TYPE_CHECKING, Any, Callable
 
 from ...compile.utils import convention_jmc_to_mc, is_float
 from ...compile.datapack_data import Item
-from .utils import ArgType, FormattedText, NumberType, find_scoreboard_player_type, verify_args, Arg
+from .utils import ArgType, FormattedText, NumberType, find_scoreboard_player_type, hash_string_to_string, verify_args, Arg
 from ..datapack import DataPack, Function
 from ..exception import JMCDecodeJSONError, JMCMissingValueError, JMCValueError
 from ..tokenizer import Token, TokenType, Tokenizer
@@ -184,16 +184,16 @@ class JMCFunction:
         """
 
     def create_new_item(
-            self, item: "Item", modify_nbt: dict[str, Token] | None = None) -> "Item":
+            self, item: "Item", modify_nbt: dict[str, Token] | None = None, error_token: Token | None = None) -> "Item":
         item_type = item.item_type
-        nbt = item.raw_nbt
+        nbt = dict(item.raw_nbt)
         if modify_nbt is None:
             modify_nbt = {}
         for key, value_token in modify_nbt.items():
             if key in nbt:
                 raise JMCValueError(
                     f"{key} is already inside the nbt",
-                    value_token,
+                    value_token if error_token is None else error_token,
                     self.tokenizer)
 
             nbt[key] = value_token
@@ -203,6 +203,39 @@ class JMCFunction:
             nbt
         )
 
+    def add_event(self, criteria: str, command: str) -> None:
+        """
+        Add command that'll run on criteria event
+
+        :param criteria: Minecraft criteria
+        :param command: Command to run
+        """
+        self.add_events(criteria, [command])
+
+    def add_events(self, criteria: str, commands: list[str]) -> None:
+        """
+        Add multiple commands that'll run on criteria event
+
+        :param criteria: Minecraft criteria
+        :param commands: Commands to run
+        """
+        criteria = criteria.replace("minecraft.", "")
+        count = criteria.lower().replace(":", "_")
+        if self.is_never_used("on_event", parameters=[criteria]):
+            objective = f"on_event_{hash_string_to_string(criteria, 7)}"
+            self.datapack.add_objective(
+                objective, criteria)
+            func_call = self.datapack.add_raw_private_function(
+                "on_event", [f"scoreboard players set @s {objective} 0", *commands], count=count)
+            self.datapack.add_tick_command(
+                f"execute as @a[scores={{{objective}=1..}}] at @s run {func_call}")
+
+        else:
+            func = self.datapack.private_functions["on_event"][count]
+            func.extend(
+                commands
+            )
+
     def create_item(self, item_type_param: str = "itemType", display_name_param: str = "displayName",
                     lore_param: str = "lore", nbt_param: str = "nbt", modify_nbt: dict[str, Token] | None = None) -> "Item":
         if modify_nbt is None:
@@ -212,7 +245,7 @@ class JMCFunction:
         if item_type.startswith("minecraft:"):
             item_type = item_type[10:]
         if self.args[lore_param]:
-            lores = self.datapack.parse_list(
+            lores, _ = self.datapack.parse_list(
                 self.raw_args[lore_param].token, self.tokenizer, TokenType.STRING)
         else:
             lores = []
@@ -295,9 +328,9 @@ class JMCFunction:
         """
         if call_string is None:
             call_string = self.call_string
-        is_not_in = call_string not in self.datapack.used_command
         if parameters is not None:
             call_string = call_string + '/' + '/'.join(parameters)
+        is_not_in = call_string not in self.datapack.used_command
         self.datapack.used_command.add(call_string)
         return is_not_in
 
