@@ -1,13 +1,14 @@
+"""Module containing all jmc exceptions"""
 from json import JSONDecodeError
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Callable
 from .log import Logger
 
 if TYPE_CHECKING:
     from .tokenizer import Token, Tokenizer
 
 logger = Logger(__name__)
-NEW_LINE = '\n'
+NEW_LINE = "\n"
 
 
 def log(self: object, args: tuple):
@@ -21,7 +22,7 @@ def log(self: object, args: tuple):
 
 
 def error_msg(message: str, token: "Token|None", tokenizer: "Tokenizer", col_length: bool,
-              display_col_length: bool, entire_line: bool, suggestion: str | None) -> str:
+              display_col_length: bool, entire_line: bool, suggestion: str | None, overide_file_str: Callable[[str], str] = lambda string: string) -> str:
     """
     Generate error message
 
@@ -34,14 +35,13 @@ def error_msg(message: str, token: "Token|None", tokenizer: "Tokenizer", col_len
     :param suggestion: A suggestion message at the end of the error message
     :return: Error message
     """
-
     if token is None:
         string = ""
         length = 1
-        col = tokenizer.col
+        col = tokenizer.col - 1
         line = tokenizer.line
     else:
-        string = token.string
+        string = token.get_full_string()
         length = token.length
         col = token.col
         line = token.line
@@ -50,24 +50,44 @@ def error_msg(message: str, token: "Token|None", tokenizer: "Tokenizer", col_len
     display_col = col
 
     if col_length:
-        if '\n' in string:
-            line += string.count('\n')
-            col = length - string.rfind('\n')
+        if "\n" in string:
+            line += string.count("\n")
+            col = length - string.rfind("\n")
         else:
             col += length
-
+        col -= 1
     if display_col_length:
-        if '\n' in string:
-            display_line += string.count('\n')
-            display_col = length - string.rfind('\n')
+        if "\n" in string:
+            display_line += string.count("\n")
+            display_col = length - string.rfind("\n")
         else:
             display_col += length
-    if entire_line:
-        msg = f"In {tokenizer.file_path}\n{message} at line {line}.\n{tokenizer.file_string.split(NEW_LINE)[display_line-1]} <-"
     else:
-        msg = f"In {tokenizer.file_path}\n{message} at line {line} col {col}.\n{tokenizer.file_string.split(NEW_LINE)[display_line-1][:display_col-1]} <-"
+        display_col += 1
+    try:
+        msgs_ = tokenizer.file_string.split(NEW_LINE)
+        max_space = len(str(display_line + 1))
+        if entire_line:
+            line_ = overide_file_str(msgs_[display_line - 1])
+            msg = f"""In {tokenizer.file_path}
+{message} at line {line}.
+{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2] if display_line > 1 else ""}
+{display_line}{" "*(max_space-len(str(display_line)))} |{line_}
+{" "*(col+max_space+1)}{"^"*(len(line_)-col+1)}
+{display_line+1} |{msgs_[display_line] if display_line < len(msgs_) else ""}"""
+        else:
+            msg = f"""In {tokenizer.file_path}
+{message} at line {line} col {col}.
+{display_line-1}{" "*(max_space-len(str(display_line - 1)))} |{msgs_[display_line-2] if display_line > 1 else ""}
+{display_line}{" "*(max_space-len(str(display_line)))} |{overide_file_str(msgs_[display_line-1])}
+{" "*(col+max_space+1)}{"^"*(display_col-col)}
+{display_line+1} |{msgs_[display_line] if display_line < len(msgs_) else ""}"""
+    except IndexError as error:
+        logger.critical(
+            f"Error happens at wrong file: {tokenizer.file_path=}, {line=}, {col=}")
+        raise error
     if suggestion is not None:
-        msg += '\n' + suggestion
+        msg += "\n" + suggestion
     return msg
 
 
@@ -92,8 +112,11 @@ class HeaderDuplicatedMacro(ValueError):
 class HeaderSyntaxException(SyntaxError):
     """Invalid syntax for header"""
 
-    def __init__(self, message: str, file_name: str, line: int, line_str: str):
+    def __init__(self, message: str, file_name: str, line: int,
+                 line_str: str, suggestion: str | None = None):
         msg = f"In {file_name}\n{message} at line {line}\n{line_str}"
+        if suggestion is not None:
+            msg += "\n" + suggestion
         log(self, (message, ))
         super().__init__(msg)
 
@@ -103,11 +126,27 @@ class JMCSyntaxException(SyntaxError):
 
     def __init__(self, message: str, token: "Token|None", tokenizer: "Tokenizer", *, col_length: bool = False,
                  display_col_length: bool = True, entire_line: bool = False, suggestion: str | None = None) -> None:
+        self.message = message
+        self.token = token
+        self.tokenizer = tokenizer
+        self.col_length = col_length
+        self.display_col_length = display_col_length
+        self.entire_line = entire_line
+        self.suggestion = suggestion
         msg = error_msg(message, token, tokenizer, col_length,
                         display_col_length, entire_line, suggestion)
         log(self, (msg, ))
         super().__init__(msg)
 
+    def reinit(self, overide_file_str: Callable[[str], str]):
+        self.msg = error_msg(self.message, self.token, self.tokenizer, self.col_length,
+                             self.display_col_length, self.entire_line, self.suggestion, overide_file_str)
+        """
+        Overide initiation
+        """
+        logger.warning("Overiding file string")
+        log(self, (self.msg, ))
+
 
 class JMCValueError(ValueError):
     """Invalid syntax for JMC"""
