@@ -44,9 +44,7 @@ FIRST_ARGUMENTS = {
     "@import",
     "new"
 }
-"""Set of all vanilla commands and JMC custom syntax
-- `if` is excluded since it can be used in execute
-- `give` is exluced since it can also be arguments (`/effect give`)"""
+"""Set of all vanilla commands and JMC custom syntax"""
 
 FIRST_ARGUMENTS_EXCEPTION = {
     "give": {"effect", "recipe", "give"},
@@ -57,9 +55,13 @@ FIRST_ARGUMENTS_EXCEPTION = {
     "bossbar": {"execute"},
     "title": {"title"},
     "item": {"summon"},
-    "loot": {"loot"}
+    "loot": {"loot"},
+    "data": {"execute"},
+    "playsound": {"weather"}
 }
-"""Dictionary of (FIRST_ARGUMENTS that can also be used as normal argument in a command) and (those commands)"""
+"""Dictionary of (FIRST_ARGUMENTS that can also be used as normal argument in a command) and (those commands)
+
+Example: `/effect clear` -> `"clear": {"effect"}`"""
 
 
 # ALLOW_KEYWORD_AFTER_CURLY_PAREN = {
@@ -67,7 +69,7 @@ FIRST_ARGUMENTS_EXCEPTION = {
 #     "clear",
 #     "setblock"
 # }
-# """Set of vanilla command to stop JMC from terminating line from curly parenthesis (Allow number after curly parenthesis)"""
+# """Set of vanilla command to stop JMC from terminating line from curly bracket (Allow number after curly bracket)"""
 
 
 def append_commands(commands: list[str], string: str) -> None:
@@ -244,7 +246,8 @@ class FuncContent:
                 and
                 self.__commands[command_pos] in FIRST_ARGUMENTS_EXCEPTION[token.string]
             ) and
-            not is_connected(token, self.command[key_pos - 1])
+            not is_connected(token, self.command[key_pos - 1]) and
+            token.string not in Header().dels
         ):
             raise JMCSyntaxException(
                 f"Keyword({token.string}) at line {token.line} col {token.col} is recognized as a command.\nExpected semicolon(;)", self.command[key_pos - 1], self.tokenizer, col_length=True)
@@ -336,11 +339,11 @@ class FuncContent:
                                                              1].token_type == TokenType.PAREN_ROUND:
             if len(self.command[key_pos:]) > 2:
                 raise JMCSyntaxException(
-                    f"Unexpected token({self.command[key_pos+2].string}) after function call. Expected semicolon(;)", self.command[key_pos + 1], self.tokenizer, col_length=True)
+                    f"Unexpected token({self.command[key_pos+2].string}) after function call. Expected semicolon(;)", self.command[key_pos], self.tokenizer, col_length=True)
 
             if self.command[key_pos + 1].string != "()":
                 raise JMCSyntaxException(
-                    f"Custom function({token.string})'s argument is not supported.\nExpected empty bracket", self.command[key_pos + 1], self.tokenizer)
+                    f"Argument is not supported in custom function({token.string}).\nExpected 0 argument, `()`", self.command[key_pos], self.tokenizer, suggestion="You might have misspelled the built-in function name. (It is case-sensitive.)")
             append_commands(self.__commands,
                             f"function {self.lexer.datapack.namespace}:{convention_jmc_to_mc(token, self.tokenizer)}")
             return True
@@ -399,11 +402,11 @@ class FuncContent:
     def __is_say(self, key_pos: int, token: Token) -> None:
         if len(self.command[key_pos:]) == 1:
             raise JMCSyntaxException(
-                "Expected string after 'say' command", token, self.tokenizer, col_length=True)
+                "Expected string after 'say' command", token, self.tokenizer)
 
         if self.command[key_pos + 1].token_type != TokenType.STRING:
             raise JMCSyntaxException(
-                "Expected string after 'say' command", self.command[key_pos + 1], self.tokenizer, display_col_length=False, suggestion="(In JMC, you are required to wrapped say command's argument in quote.)")
+                "Expected string after 'say' command", self.command[key_pos + 1], self.tokenizer, suggestion="(In JMC, you are required to wrapped say command's argument in quote.)")
 
         if len(self.command[key_pos:]) > 2:
             raise JMCSyntaxException(
@@ -453,7 +456,7 @@ class FuncContent:
                 return True
             if len(self.command) < key_pos + 5:
                 raise JMCSyntaxException(
-                    f"Expected time in schedule call", self.command[key_pos + 4], self.tokenizer)
+                    f"Expected time in schedule call", self.command[key_pos + 3], self.tokenizer)
             append_commands(self.__commands, self.command[key_pos + 4].string)
             if len(self.command) == key_pos + 6:
                 if self.command[key_pos +
@@ -538,7 +541,7 @@ class FuncContent:
         if jmc_command is not None:
             if len(self.command) > key_pos + 2:
                 raise JMCSyntaxException(
-                    "Unexpected token", self.command[key_pos + 2], self.tokenizer, display_col_length=False)
+                    "Unexpected token", self.command[key_pos + 2], self.tokenizer, display_col_length=False, suggestion="Expected semicolon")
             append_commands(self.__commands, jmc_command(
                 self.command[key_pos + 1], self.lexer.datapack, self.tokenizer, is_execute=self.is_execute).call())
             return True
