@@ -1,17 +1,18 @@
+"""Module for testing compilation"""
 from json import dumps
 from pathlib import Path
 
 from ..terminal.configuration import Configuration, GlobalData
 from .log import Logger
 from .header import Header
-from .compile import read_cert, read_header, build
+from .compiling import read_cert, read_header, build
 from .lexer import Lexer
 
 
 logger = Logger(__name__)
 
 
-class JMCPack:
+class JMCTestPack:
     """
     Class representation of folder structure containing entire JMC files
 
@@ -20,8 +21,8 @@ class JMCPack:
     :param namespace: Namespace of the virtual datapack, defaults to "TEST"
     :param output: virtual directory for output, defaults to "VIRTUAL"
     """
-    __slots__ = ('cert', 'jmc_file', 'header_file',
-                 '__built', 'config')
+    __slots__ = ("cert", "jmc_file", "header_file",
+                 "__built", "config")
     cert: str
     jmc_file: str
     header_file: str | None
@@ -43,29 +44,61 @@ INT=__int__"""
             namespace=namespace,
             description="__THIS_IS_FOR_TESTING__",
             pack_format="10",
-            target=Path(__file__),
+            target=Path("main.jmc"),
             output=Path(output)
         )
 
-    def set_cert(self, file_content: str) -> "JMCPack":
+    def set_cert(self, file_content: str) -> "JMCTestPack":
+        """
+        Set certificate(jmc.txt)
+
+        :param file_content: File's content
+        :return: Self
+        """
         self.cert = file_content
         return self
 
-    def set_jmc_file(self, file_content: str) -> "JMCPack":
+    def set_jmc_file(self, file_content: str) -> "JMCTestPack":
+        """
+        Set main jmc file
+
+        :param file_content: File's content
+        :return: Self
+        """
         self.jmc_file = file_content
         return self
 
-    def set_header_file(self, file_content: str) -> "JMCPack":
+    def set_header_file(self, file_content: str) -> "JMCTestPack":
+        """
+        Set main header file
+
+        :param file_content: File's content
+        :return: Self
+        """
         self.header_file = file_content
         return self
 
-    def build(self) -> "JMCPack":
+    def build(self) -> "JMCTestPack":
+        """
+        Build datapack
+        :return: Self
+        """
         logger.info("Building from JMCPack")
         Header.clear()
-        read_cert(self.config, _test_file=self.cert)
+        is_delete, cert_config, cert_file = read_cert(
+            self.config, _test_file=self.cert)
         read_header(self.config, _test_file=self.header_file)
         lexer = Lexer(self.config, _test_file=self.jmc_file)
-        self.__built = build(lexer.datapack, self.config, _is_virtual=True)
+        built = build(
+            lexer.datapack,
+            self.config,
+            is_delete, cert_config, cert_file,
+            _is_virtual=True)
+        if built is None:
+            raise ValueError
+        self.__built = {
+            path.as_posix(): content for path,
+            content in built.items()}
         return self
 
     @property
