@@ -1,22 +1,23 @@
 from enum import Enum
 from getpass import getpass
 from threading import Event
+from traceback import format_exc
 from ..compile import Logger
 
 logger = Logger(__name__)
 
 
 class Colors(Enum):
-    HEADER = '\033[1;33;40m'
-    YELLOW = '\033[33;40m'
-    INFO = '\033[94;40m'
-    INPUT = '\033[96;40m'
-    PURPLE = '\033[35;40m'
-    FAIL = '\033[91;40m'
-    FAIL_BOLD = '\033[1;91;40m'
-    ENDC = '\033[0;0;40m'
-    EXIT = '\033[0;0;0m'
-    NONE = '\033[0;37;40m'
+    HEADER = "\033[1;33;40m"
+    YELLOW = "\033[33;40m"
+    INFO = "\033[94;40m"
+    INPUT = "\033[96;40m"
+    PURPLE = "\033[35;40m"
+    FAIL = "\033[91;40m"
+    FAIL_BOLD = "\033[1;91;40m"
+    ENDC = "\033[0;0;40m"
+    EXIT = "\033[0;0;0m"
+    NONE = "\033[0;37;40m"
 
 
 def pprint(values, color: Colors = Colors.NONE):
@@ -68,14 +69,17 @@ def handle_exception(error: Exception, event: Event, is_ok: bool):
     Tell user when unexpected crash happens and reset
 
     :param error: Exception
+    :param event: Event object to stop compiling
+    :param is_ok: Whether to tell user this error is normal
     """
     event.set()
     pprint("Unexpected error causes program to crash", Colors.FAIL)
     pprint(type(error).__name__, Colors.FAIL_BOLD)
     pprint(error, Colors.FAIL)
     if not is_ok:
+        pprint(format_exc(), Colors.YELLOW)
         pprint(
-            "NOTE: This shouldn't happen. Please contact WingedSeal.",
+            "Please report this error at https://github.com/WingedSeal/jmc/issues/new/choose or https://discord.gg/PNWKpwdzD3.",
             Colors.FAIL)
     logger.critical("Program crashed")
     logger.exception("")
